# thermals.py
#
# Copyright 2025 Stephen Horvath
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# SPDX-License-Identifier: GPL-2.0-or-later

from gi.repository import Adw
from gi.repository import Gtk
from gi.repository import GLib

import cros_ec_python.commands as ec_commands
import cros_ec_python.exceptions as ec_exceptions

@Gtk.Template(resource_path='/au/stevetech/yafi/ui/thermals.ui')
class ThermalsPage(Gtk.Box):
    __gtype_name__ = 'ThermalsPage'

    fan_rpm = Gtk.Template.Child()
    fan_mode = Gtk.Template.Child()
    fan_set_rpm = Gtk.Template.Child()
    fan_set_percent = Gtk.Template.Child()
    fan_percent_scale = Gtk.Template.Child()

    temperatures = Gtk.Template.Child()
    temp_items = []

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def setup(self, app):
        # Don't let the user change the fans if they can't get back to auto
        if ec_commands.general.get_cmd_versions(
            app.cros_ec, ec_commands.thermal.EC_CMD_THERMAL_AUTO_FAN_CTRL
        ):

            def handle_fan_mode(mode):
                match mode:
                    case 0:  # Auto
                        self.fan_set_rpm.set_visible(False)
                        self.fan_set_percent.set_visible(False)
                        ec_commands.thermal.thermal_auto_fan_ctrl(app.cros_ec)
                    case 1:  # Percent
                        self.fan_set_rpm.set_visible(False)
                        self.fan_set_percent.set_visible(True)
                    case 2:  # RPM
                        self.fan_set_rpm.set_visible(True)
                        self.fan_set_percent.set_visible(False)

            self.fan_mode.connect(
                "notify::selected",
                lambda combo, _: handle_fan_mode(combo.get_selected()),
            )

            if ec_commands.general.get_cmd_versions(
                app.cros_ec, ec_commands.pwm.EC_CMD_PWM_SET_FAN_DUTY
            ):

                def handle_fan_percent(scale):
                    percent = int(scale.get_value())
                    ec_commands.pwm.pwm_set_fan_duty(app.cros_ec, percent)
                    self.fan_set_percent.set_subtitle(f"{percent} %")

                self.fan_percent_scale.connect("value-changed", handle_fan_percent)
            else:
                self.fan_set_percent.set_sensitive(False)

            if ec_commands.general.get_cmd_versions(
                app.cros_ec, ec_commands.pwm.EC_CMD_PWM_SET_FAN_TARGET_RPM
            ):

                def handle_fan_rpm(entry):
                    rpm = int(entry.get_text())
                    ec_commands.pwm.pwm_set_fan_rpm(app.cros_ec, rpm)

                self.fan_set_rpm.connect(
                    "notify::text", lambda entry, _: handle_fan_rpm(entry)
                )
            else:
                self.fan_set_rpm.set_sensitive(False)
        else:
            self.fan_mode.set_sensitive(False)

        # Temperature sensors
        while temp_child := self.temperatures.get_last_child():
            self.temperatures.remove(temp_child)
        self.temp_items.clear()

        try:
            ec_temp_sensors = ec_commands.thermal.get_temp_sensors(app.cros_ec)
        except ec_exceptions.ECError as e:
            if e.ec_status == ec_exceptions.EcStatus.EC_RES_INVALID_COMMAND:
                # Generate some labels if the command is not supported
                ec_temp_sensors = {}
                temps = ec_commands.memmap.get_temps(app.cros_ec)
                for i, temp in enumerate(temps):
                    ec_temp_sensors[f"Sensor {i}"] = (temp, None)
            else:
                raise e

        for key, value in ec_temp_sensors.items():
            new_row = Adw.ActionRow(title=key, subtitle=f"{value[0]}°C")
            new_row.add_css_class("property")
            self.temperatures.append(new_row)
            self.temp_items.append(new_row)

        self._update_thermals(app)

        # Schedule _update_thermals to run every second
        GLib.timeout_add_seconds(1, self._update_thermals, app)

    def _update_thermals(self, app):
        # memmap reads should always be supported
        ec_fans = ec_commands.memmap.get_fans(app.cros_ec)
        self.fan_rpm.set_subtitle(f"{ec_fans[0]} RPM")

        ec_temp_sensors = ec_commands.memmap.get_temps(app.cros_ec)
        # The temp sensors disappear sometimes, so we need to handle that
        for i in range(min(len(self.temp_items), len(ec_temp_sensors))):
            self.temp_items[i].set_subtitle(f"{ec_temp_sensors[i]}°C")

        # Check if this has already failed and skip if it has
        if not ec_commands.pwm.EC_CMD_PWM_GET_FAN_TARGET_RPM in app.no_support:
            try:
                ec_target_rpm = ec_commands.pwm.pwm_get_fan_rpm(app.cros_ec)
                self.fan_set_rpm.set_subtitle(f"{ec_target_rpm} RPM")
            except ec_exceptions.ECError as e:
                # If the command is not supported, we can ignore it
                if e.ec_status == ec_exceptions.EcStatus.EC_RES_INVALID_COMMAND:
                    app.no_support.append(ec_commands.pwm.EC_CMD_PWM_GET_FAN_TARGET_RPM)
                    self.fan_set_rpm.set_subtitle("")
                else:
                    # If it's another error, we should raise it
                    raise e

        return app.current_page == 0



import os

# Register resources if not running in Flatpak
if not os.environ.get('FLATPAK_ID'):
    from gi.repository import Gio
    resource = Gio.Resource.load(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'yafi.gresource'))
    resource._register()

from .main import main



# hardware.py
#
# Copyright 2025 Stephen Horvath
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# SPDX-License-Identifier: GPL-2.0-or-later

from gi.repository import Adw
from gi.repository import Gtk
from gi.repository import GLib

import cros_ec_python.commands as ec_commands
import cros_ec_python.exceptions as ec_exceptions

@Gtk.Template(resource_path='/au/stevetech/yafi/ui/hardware.ui')
class HardwarePage(Gtk.Box):
    __gtype_name__ = 'HardwarePage'

    hw_chassis = Gtk.Template.Child()
    hw_chassis_label = Gtk.Template.Child()

    hw_priv_cam = Gtk.Template.Child()
    hw_priv_cam_sw = Gtk.Template.Child()
    hw_priv_mic = Gtk.Template.Child()
    hw_priv_mic_sw = Gtk.Template.Child()

    hw_fp_pwr = Gtk.Template.Child()
    hw_fp_pwr_en = Gtk.Template.Child()
    hw_fp_pwr_dis = Gtk.Template.Child()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def setup(self, app):
        # Fingerprint Power (Untested)
        if ec_commands.general.get_cmd_versions(
            app.cros_ec, ec_commands.framework_laptop.EC_CMD_FP_CONTROL
        ):
            self.hw_fp_pwr_en.connect(
                "clicked",
                lambda _: ec_commands.framework_laptop.fp_control(app.cros_ec, True),
            )

            self.hw_fp_pwr_dis.connect(
                "clicked",
                lambda _: ec_commands.framework_laptop.fp_control(app.cros_ec, False),
            )
            self.hw_fp_pwr.set_visible(True)
        else:
            self.hw_fp_pwr.set_visible(False)

        self._update_hardware(app)

        # Schedule _update_hardware to run every second
        GLib.timeout_add_seconds(1, self._update_hardware, app)

    def _update_hardware(self, app):
        success = False
        # Chassis
        if not ec_commands.framework_laptop.EC_CMD_CHASSIS_INTRUSION in app.no_support:
            try:
                ec_chassis = ec_commands.framework_laptop.get_chassis_intrusion(
                    app.cros_ec
                )

                self.hw_chassis_label.set_label(str(ec_chassis["total_open_count"]))

                ec_chassis_open = ec_commands.framework_laptop.get_chassis_open_check(
                    app.cros_ec
                )
                self.hw_chassis.set_subtitle(
                    "Currently " + ("Open" if ec_chassis_open else "Closed")
                )

                success = True
            except ec_exceptions.ECError as e:
                if e.ec_status == ec_exceptions.EcStatus.EC_RES_INVALID_COMMAND:
                    app.no_support.append(
                        ec_commands.framework_laptop.EC_CMD_CHASSIS_INTRUSION
                    )
                    self.hw_chassis.set_visible(False)
                else:
                    raise e

        # Privacy Switches
        if (
            not ec_commands.framework_laptop.EC_CMD_PRIVACY_SWITCHES_CHECK_MODE
            in app.no_support
        ):
            try:
                ec_privacy = ec_commands.framework_laptop.get_privacy_switches(
                    app.cros_ec
                )
                self.hw_priv_cam_sw.set_active(ec_privacy["camera"])
                self.hw_priv_mic_sw.set_active(ec_privacy["microphone"])

                success = True
            except ec_exceptions.ECError as e:
                if e.ec_status == ec_exceptions.EcStatus.EC_RES_INVALID_COMMAND:
                    app.no_support.append(
                        ec_commands.framework_laptop.EC_CMD_PRIVACY_SWITCHES_CHECK_MODE
                    )
                    self.hw_priv_cam.set_visible(False)
                    self.hw_priv_mic.set_visible(False)
                else:
                    raise e

        return app.current_page == 3 and success



# leds.py
#
# Copyright 2025 Stephen Horvath
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# SPDX-License-Identifier: GPL-2.0-or-later

from gi.repository import Adw
from gi.repository import Gtk

import cros_ec_python.commands as ec_commands
import cros_ec_python.exceptions as ec_exceptions

@Gtk.Template(resource_path='/au/stevetech/yafi/ui/leds.ui')
class LedsPage(Gtk.Box):
    __gtype_name__ = 'LedsPage'

    led_pwr = Gtk.Template.Child()
    led_pwr_scale = Gtk.Template.Child()

    led_kbd = Gtk.Template.Child()
    led_kbd_scale = Gtk.Template.Child()

    led_advanced = Gtk.Template.Child()

    led_pwr_colour = Gtk.Template.Child()

    led_chg_colour = Gtk.Template.Child()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def setup(self, app):
        # Power LED
        try:

            def handle_led_pwr(scale):
                value = int(abs(scale.get_value() - 2))
                ec_commands.framework_laptop.set_fp_led_level(app.cros_ec, value)
                self.led_pwr.set_subtitle(["High", "Medium", "Low"][value])

            current_fp_level = ec_commands.framework_laptop.get_fp_led_level(
                app.cros_ec
            ).value
            self.led_pwr_scale.set_value(abs(current_fp_level - 2))
            self.led_pwr.set_subtitle(["High", "Medium", "Low"][current_fp_level])
            self.led_pwr_scale.connect("value-changed", handle_led_pwr)
        except ec_exceptions.ECError as e:
            if e.ec_status == ec_exceptions.EcStatus.EC_RES_INVALID_COMMAND:
                app.no_support.append(ec_commands.framework_laptop.EC_CMD_FP_LED_LEVEL)
                self.led_pwr.set_visible(False)
            else:
                raise e

        # Keyboard backlight
        if ec_commands.general.get_cmd_versions(
            app.cros_ec, ec_commands.pwm.EC_CMD_PWM_SET_KEYBOARD_BACKLIGHT
        ):

            def handle_led_kbd(scale):
                value = int(scale.get_value())
                ec_commands.pwm.pwm_set_keyboard_backlight(app.cros_ec, value)
                self.led_kbd.set_subtitle(f"{value} %")

            current_kb_level = ec_commands.pwm.pwm_get_keyboard_backlight(app.cros_ec)[
                "percent"
            ]
            self.led_kbd_scale.set_value(current_kb_level)
            self.led_kbd.set_subtitle(f"{current_kb_level} %")
            self.led_kbd_scale.connect("value-changed", handle_led_kbd)
        else:
            self.led_kbd.set_visible(False)

        # Advanced options
        if ec_commands.general.get_cmd_versions(
            app.cros_ec, ec_commands.leds.EC_CMD_LED_CONTROL
        ):

            # Advanced: Power LED
            led_pwr_colour_strings = self.led_pwr_colour.get_model()

            all_colours = ["Red", "Green", "Blue", "Yellow", "White", "Amber"]

            def add_colours(strings, led_id):
                # Auto and Off should already be present
                if strings.get_n_items() <= 2:
                    supported_colours = ec_commands.leds.led_control_get_max_values(
                        app.cros_ec, led_id
                    )
                    for i, colour in enumerate(all_colours):
                        if supported_colours[i]:
                            strings.append(colour)

            add_colours(
                led_pwr_colour_strings, ec_commands.leds.EcLedId.EC_LED_ID_POWER_LED
            )

            def handle_led_colour(combobox, led_id):
                colour = combobox.get_selected() - 2
                match colour:
                    case -2:  # Auto
                        ec_commands.leds.led_control_set_auto(app.cros_ec, led_id)
                    case -1:  # Off
                        ec_commands.leds.led_control(
                            app.cros_ec,
                            led_id,
                            0,
                            [0] * ec_commands.leds.EcLedColors.EC_LED_COLOR_COUNT.value,
                        )
                    case _:  # Colour
                        colour_idx = all_colours.index(
                            combobox.get_selected_item().get_string()
                        )
                        ec_commands.leds.led_control_set_color(
                            app.cros_ec,
                            led_id,
                            100,
                            ec_commands.leds.EcLedColors(colour_idx),
                        )

            self.led_pwr_colour.connect(
                "notify::selected",
                lambda combo, _: handle_led_colour(
                    combo, ec_commands.leds.EcLedId.EC_LED_ID_POWER_LED
                ),
            )

            # Advanced: Charging LED
            led_chg_colour_strings = self.led_chg_colour.get_model()
            
            add_colours(
                led_chg_colour_strings,
                ec_commands.leds.EcLedId.EC_LED_ID_BATTERY_LED,
            )

            self.led_chg_colour.connect(
                "notify::selected",
                lambda combo, _: handle_led_colour(
                    combo, ec_commands.leds.EcLedId.EC_LED_ID_BATTERY_LED
                ),
            )
        else:
            self.led_advanced.set_visible(False)



# window.py
#
# Copyright 2025 Stephen Horvath
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# SPDX-License-Identifier: GPL-2.0-or-later

from gi.repository import Adw
from gi.repository import Gtk

@Gtk.Template(resource_path='/au/stevetech/yafi/ui/yafi.ui')
class YafiWindow(Adw.ApplicationWindow):
    __gtype_name__ = 'YafiWindow'

    content = Gtk.Template.Child()
    navbar = Gtk.Template.Child()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)



# battery.py
#
# Copyright 2025 Stephen Horvath
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# SPDX-License-Identifier: GPL-2.0-or-later

from gi.repository import Adw
from gi.repository import Gtk
from gi.repository import GLib

import cros_ec_python.commands as ec_commands
import cros_ec_python.exceptions as ec_exceptions

@Gtk.Template(resource_path='/au/stevetech/yafi/ui/battery.ui')
class BatteryPage(Gtk.Box):
    __gtype_name__ = 'BatteryPage'

    chg_limit_enable = Gtk.Template.Child()
    chg_limit = Gtk.Template.Child()
    chg_limit_scale = Gtk.Template.Child()
    bat_limit = Gtk.Template.Child()
    bat_limit_scale = Gtk.Template.Child()
    chg_limit_override = Gtk.Template.Child()
    chg_limit_override_btn = Gtk.Template.Child()

    bat_ext_group = Gtk.Template.Child()
    bat_ext_enable = Gtk.Template.Child()
    bat_ext_stage = Gtk.Template.Child()
    bat_ext_trigger_time = Gtk.Template.Child()
    bat_ext_reset_time = Gtk.Template.Child()
    bat_ext_trigger = Gtk.Template.Child()
    bat_ext_reset = Gtk.Template.Child()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def setup(self, app):
        # Charge limiter
        try:
            ec_limit = ec_commands.framework_laptop.get_charge_limit(app.cros_ec)
            ec_limit_enabled = ec_limit != (0, 0)
            self.chg_limit_enable.set_active(ec_limit_enabled)
            if ec_limit_enabled:
                self.chg_limit_scale.set_value(ec_limit[0])
                self.bat_limit_scale.set_value(ec_limit[1])
                self.chg_limit.set_sensitive(True)
                self.bat_limit.set_sensitive(True)
                self.chg_limit_override.set_sensitive(True)

            def handle_chg_limit_change(min, max):
                ec_commands.framework_laptop.set_charge_limit(
                    app.cros_ec, int(min), int(max)
                )

            def handle_chg_limit_enable(switch):
                active = switch.get_active()
                if active:
                    handle_chg_limit_change(
                        self.chg_limit_scale.get_value(), self.bat_limit_scale.get_value()
                    )
                else:
                    ec_commands.framework_laptop.disable_charge_limit(app.cros_ec)

                self.chg_limit.set_sensitive(active)
                self.bat_limit.set_sensitive(active)
                self.chg_limit_override.set_sensitive(active)

            self.chg_limit_enable.connect(
                "notify::active", lambda switch, _: handle_chg_limit_enable(switch)
            )
            self.chg_limit_scale.connect(
                "value-changed",
                lambda scale: handle_chg_limit_change(
                    scale.get_value(), self.bat_limit_scale.get_value()
                ),
            )
            self.bat_limit_scale.connect(
                "value-changed",
                lambda scale: handle_chg_limit_change(
                    self.chg_limit_scale.get_value(), scale.get_value()
                ),
            )

            self.chg_limit_override_btn.connect(
                "clicked",
                lambda _: ec_commands.framework_laptop.override_charge_limit(
                    app.cros_ec
                ),
            )
        except ec_exceptions.ECError as e:
            if e.ec_status == ec_exceptions.EcStatus.EC_RES_INVALID_COMMAND:
                app.no_support.append(ec_commands.framework_laptop.EC_CMD_CHARGE_LIMIT)
                self.chg_limit_enable.set_sensitive(False)
            else:
                raise e

        # Battery Extender
        try:
            ec_extender = ec_commands.framework_laptop.get_battery_extender(
                app.cros_ec
            )
            self.bat_ext_enable.set_active(not ec_extender["disable"])
            self.bat_ext_stage.set_sensitive(not ec_extender["disable"])
            self.bat_ext_trigger_time.set_sensitive(not ec_extender["disable"])
            self.bat_ext_reset_time.set_sensitive(not ec_extender["disable"])
            self.bat_ext_trigger.set_sensitive(not ec_extender["disable"])
            self.bat_ext_reset.set_sensitive(not ec_extender["disable"])

            self.bat_ext_stage.set_subtitle(str(ec_extender["current_stage"]))
            self.bat_ext_trigger_time.set_subtitle(
                format_timedelta(ec_extender["trigger_timedelta"])
            )
            self.bat_ext_reset_time.set_subtitle(
                format_timedelta(ec_extender["reset_timedelta"])
            )
            self.bat_ext_trigger.set_value(ec_extender["trigger_days"])
            self.bat_ext_reset.set_value(ec_extender["reset_minutes"])

            def handle_extender_enable(switch):
                active = switch.get_active()
                ec_commands.framework_laptop.set_battery_extender(
                    app.cros_ec,
                    not active,
                    int(self.bat_ext_trigger.get_value()),
                    int(self.bat_ext_reset.get_value()),
                )
                self.bat_ext_stage.set_sensitive(active)
                self.bat_ext_trigger_time.set_sensitive(active)
                self.bat_ext_reset_time.set_sensitive(active)
                self.bat_ext_trigger.set_sensitive(active)
                self.bat_ext_reset.set_sensitive(active)

            self.bat_ext_enable.connect(
                "notify::active", lambda switch, _: handle_extender_enable(switch)
            )
            self.bat_ext_trigger.connect(
                "notify::value",
                lambda scale, _: ec_commands.framework_laptop.set_battery_extender(
                    app.cros_ec,
                    not self.bat_ext_enable.get_active(),
                    int(scale.get_value()),
                    int(self.bat_ext_reset.get_value()),
                ),
            )
            self.bat_ext_reset.connect(
                "notify::value",
                lambda scale, _: ec_commands.framework_laptop.set_battery_extender(
                    app.cros_ec,
                    not self.bat_ext_enable.get_active(),
                    int(self.bat_ext_trigger.get_value()),
                    int(scale.get_value()),
                ),
            )
        except ec_exceptions.ECError as e:
            if e.ec_status == ec_exceptions.EcStatus.EC_RES_INVALID_COMMAND:
                app.no_support.append(
                    ec_commands.framework_laptop.EC_CMD_BATTERY_EXTENDER
                )
                self.bat_ext_group.set_visible(False)
            else:
                raise e

        # Schedule _update_battery to run every second
        GLib.timeout_add_seconds(
            1,
            self._update_battery,
            app
        )

    def _update_battery(self, app):
        if ec_commands.framework_laptop.EC_CMD_BATTERY_EXTENDER in app.no_support:
            return False

        try:
            ec_extender = ec_commands.framework_laptop.get_battery_extender(
                app.cros_ec
            )

            self.bat_ext_stage.set_subtitle(str(ec_extender["current_stage"]))
            self.bat_ext_trigger_time.set_subtitle(
                format_timedelta(ec_extender["trigger_timedelta"])
            )
            self.bat_ext_reset_time.set_subtitle(
                format_timedelta(ec_extender["reset_timedelta"])
            )
        except ec_exceptions.ECError as e:
            if e.ec_status == ec_exceptions.EcStatus.EC_RES_INVALID_COMMAND:
                app.no_support.append(
                    ec_commands.framework_laptop.EC_CMD_BATTERY_EXTENDER
                )
                return False
            else:
                raise e

        return app.current_page == 2

def format_timedelta(timedelta):
    days = f"{timedelta.days} days, " if timedelta.days else ""
    hours, remainder = divmod(timedelta.seconds, 3600)
    minutes, seconds = divmod(remainder, 60)
    return days + f"{hours}:{minutes:02}:{seconds:02}"



# main.py
#
# Copyright 2025 Stephen Horvath
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# SPDX-License-Identifier: GPL-2.0-or-later

import sys
import traceback
import gi
from gi.repository import Gio

gi.require_version('Gtk', '4.0')
gi.require_version('Adw', '1')

from gi.repository import Gtk, Gio, Adw
from .window import YafiWindow
from .thermals import ThermalsPage
from .leds import LedsPage
from .battery import BatteryPage
from .hardware import HardwarePage

from cros_ec_python import get_cros_ec


class YafiApplication(Adw.Application):
    """The main application singleton class."""

    def __init__(self):
        super().__init__(application_id='au.stevetech.yafi',
                         flags=Gio.ApplicationFlags.DEFAULT_FLAGS,
                         resource_base_path='/au/stevetech/yafi')

        self.current_page = 0
        self.no_support = []
        self.cros_ec = None
        self.win = None

    def change_page(self, content, page):
        page.setup(self)
        while content_child := content.get_last_child():
            content.remove(content_child)
        content.append(page)

    def do_activate(self):
        """Called when the application is activated.

        We raise the application's main window, creating it if
        necessary.
        """
        self.win = self.props.active_window
        if not self.win:
            self.win = YafiWindow(application=self)

        try:
            self.cros_ec = get_cros_ec()
            pass
        except Exception as e:
            traceback.print_exc()

            message = (
                str(e)
                + "\n\n"
                + "This application only supports Framework Laptops.\n"
                + "If you are using a Framework Laptop, there are additional troubleshooting steps in the README."
            )
            self.show_error("EC Initalisation Error", message)
            self.win.present()
            return

        self.change_page(self.win.content, ThermalsPage())

        pages = (
            ("Thermals", ThermalsPage()),
            ("LEDs", LedsPage()),
            ("Battery", BatteryPage()),
            ("Hardware", HardwarePage()),
            ("About", None),
        )

        # Build the navbar
        for page in pages:
            row = Gtk.ListBoxRow()
            row.set_child(Gtk.Label(label=page[0]))
            self.win.navbar.append(row)

        def switch_page(page):
            # About page is a special case
            if pages[page][1]:
                self.current_page = page
                self.change_page(self.win.content, pages[page][1])
            else:
                self.on_about_action()

        self.win.navbar.connect("row-activated", lambda box, row: switch_page(row.get_index()))

        self.win.present()

    def on_about_action(self, *args):
        """Callback for the app.about action."""
        about = Adw.AboutDialog(
            application_icon="au.stevetech.yafi",
            application_name="Yet Another Framework Interface",
            comments="YAFI is another GUI for the Framework Laptop Embedded Controller.\n"
            + "It is written in Python with a GTK3 theme, and uses the `CrOS_EC_Python` library to communicate with the EC.",
            copyright="© 2025 Stephen Horvath",
            developer_name="Stephen Horvath",
            developers=["Stephen Horvath"],
            issue_url="https://github.com/Steve-Tech/YAFI/issues",
            license_type=Gtk.License.GPL_2_0,
            version="0.2",
            website="https://github.com/Steve-Tech/YAFI",
        )
        about.add_acknowledgement_section(None, ["Framework Computer Inc. https://frame.work/"])
        about.present(self.props.active_window)

    def show_error(self, heading, message):
        dialog = Adw.AlertDialog(heading=heading, body=message)
        dialog.add_response("exit", "Exit")
        dialog.connect("response", lambda d, r: self.win.destroy())
        dialog.present(self.win)


def main():
    """The application's entry point."""
    app = YafiApplication()
    return app.run(sys.argv)
